package com.chinacreator.kafka.quartz;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

import org.jboss.logging.Logger;
import org.quartz.CronScheduleBuilder;
import org.quartz.CronTrigger;
import org.quartz.Job;
import org.quartz.JobBuilder;
import org.quartz.JobDataMap;
import org.quartz.JobDetail;
import org.quartz.JobKey;
import org.quartz.SchedulerException;
import org.quartz.TriggerBuilder;
import org.quartz.TriggerKey;
import org.quartz.impl.StdSchedulerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import com.chinacreator.kafka.bean.TbKafkaJobConf;
import com.chinacreator.kafka.mapper.KafkaInfoDao;
import com.chinacreator.kafka.util.ShareVarUtil;
import com.chinacreator.kafka.util.ThreadLimitUtil;

@Component
public class QuartzService {
	
	private static Logger logger = Logger.getLogger(QuartzService.class);
	
	//public static boolean flag = false;
	
	private org.quartz.Scheduler scheduler;
	
	@Value("${config.jobRunMaxTime}")
	private String jobRunMaxTime;
	
	@Autowired
	KafkaInfoDao kafkaInfoDao;
	
	public void process(){
		System.out.println(">>>>>>>>>开始创建定时任务"+new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date()));	
		try {
			//1.创建Scheduler的工厂，从工厂中获取调度器实例
			if(scheduler== null) {
				scheduler = StdSchedulerFactory.getDefaultScheduler();
			}	
			//初始化定时任务
			initJob(scheduler);
			//启动定时任务
			start(scheduler);			
		} catch (Exception e) {
			e.printStackTrace();
			logger.error("创建定时任务出现异常，"+e.getMessage());
		}
		System.out.println(">>>>>>>>>创建定时任务完成"+new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date()));	
	}
	
	/**
	 * 任务初始化
	 * 
	 * @param scheduler
	 * @throws SchedulerException
	 * @throws ClassNotFoundException
	 */
	public void initJob(org.quartz.Scheduler scheduler) throws SchedulerException, ClassNotFoundException {
		if (scheduler != null) {
			logger.info("【任务初始化】 .... ....");
			addProcessJob(scheduler);
		}
	}
	
	/**
	 * 任务初始化 - 添加所有任务
	 * 
	 * @param scheduler
	 * @throws SchedulerException
	 * @throws ClassNotFoundException
	 */
	private void addProcessJob(org.quartz.Scheduler scheduler) throws SchedulerException, ClassNotFoundException {
		logger.info("【初始化Job】开始");
		List<TbKafkaJobConf> list = kafkaInfoDao.getJobConf();
		if (list != null && list.size() > 0) {
			for (TbKafkaJobConf jobConf : list) {
				String jobStatus = jobConf.getSTATUS();
				String jobname = jobConf.getNAME();
				/**
				if (!"0".equals(jobStatus)) {
					continue;
				}
				**/			
				try {
					Class<? extends Job> jobClass = (Class<? extends Job>) Class.forName(jobConf.getHANDLERCLASSNAME());
					addJob(scheduler,jobname, jobConf.getCRONEXPRESSION(), jobClass, jobConf, jobStatus);
				} catch (Exception e) {
					e.printStackTrace();
					logger.info("初始化Job failed .... ....");
				}
			}
		} 
		logger.info("【初始化Job】结束");
	}
	
	/**
	 * 开始一个定时任务
	 * @param scheduler
	 * @param jobname
	 * @param cronexpression
	 * @param jobClass
	 * @param data
	 * @param status
	 * @throws SchedulerException
	 */
	private void addJob(org.quartz.Scheduler scheduler,String jobname, String cronexpression, Class<? extends Job> jobClass, Object data, String status)
			throws SchedulerException {
		//设置JOB的名和组
		JobKey jobKey = new JobKey(jobname, jobname);
		//logger.info("=============="+scheduler.checkExists(jobKey));
		
		boolean hasAddTaks = false; //是否添加定时任务
		//配置为禁用状态，不再生成定时任务
		if (!"0".equals(status)) {
			if (scheduler.checkExists(jobKey)) {
				scheduler.pauseJob(jobKey);
				scheduler.deleteJob(jobKey);
				logger.info("配置状态为禁用，删除定时任务【"+jobname+"】");
			}
			//移除定时任务标识
			ThreadLimitUtil.removeJobThread(jobname);
			return;
		}else if(ThreadLimitUtil.hasJobThread(jobname)){ //jobname已经存在
			logger.info("定时任务【"+jobname+"】，已经存在");
			//job大于最大运行时间，删除定时任务
			if(ThreadLimitUtil.hasGreatMaxRunTime(jobname, jobRunMaxTime)){
				logger.info("定时任务【"+jobname+"】，运行时间超过最大值【"+jobRunMaxTime+"】");
				//如果已经存在，先暂停，再删除
				if (scheduler.checkExists(jobKey)) {
					scheduler.pauseJob(jobKey);
					scheduler.deleteJob(jobKey);
					logger.info("删除定时任务【"+jobname+"】");
				}
				//移除定时任务标识
				ThreadLimitUtil.removeJobThread(jobname);
				hasAddTaks = true;
			}
		}else{ //配置为0，且定时任务未添加
			//设置定时任务标识
			hasAddTaks = true;
		}
		
		//不能添加
		if(!hasAddTaks){
			logger.info("此次不添加定时任务【"+jobname+"】");
			return;
		}else{
			ThreadLimitUtil.setJobThread(jobname);
		}
		
		logger.info("启动定时任务【"+jobname+"】  ");

		try {
			JobDataMap jobMap = new JobDataMap();
			if (data != null) {
				jobMap.put("data", data);
				jobMap.put("status", status);
			}
			//3.创建JobDetail
			JobDetail detail = JobBuilder.newJob(jobClass) 		// jobClass 为一个job,是要执行的一个任务。
					.withIdentity(jobKey) //job 的name和group
					.usingJobData(jobMap) //传递参数
					.build();
			//4.创建Trigger
			//使用SimpleScheduleBuilder或者CronScheduleBuilder
			CronTrigger trigger = TriggerBuilder.newTrigger()
					.startNow() //马上启动
					.withIdentity(new TriggerKey(jobname + "trigger", jobname))
					.withSchedule(CronScheduleBuilder.cronSchedule(cronexpression)).build();
			//5.注册任务和定时器
			scheduler.scheduleJob(detail, trigger);
		} catch (Exception e) {
			//移除定时任务标识
			ThreadLimitUtil.removeJobThread(jobname);
			throw e;
		}
	}
	
	/**
	 * 任务启动
	 * 
	 * @param scheduler
	 * @throws SchedulerException
	 */
	public void start(org.quartz.Scheduler scheduler) throws SchedulerException {
		if (scheduler != null) {
			//6.启动 调度器
			scheduler.start();
		}
	}
	
	/**
	 * 更新Job
	 * @param scheduler
	 */
	/****
	private void jobStatusUpdate(org.quartz.Scheduler scheduler) {
		logger.info("【任务更新】 .... ....");
		
		List<TbKafkaJobConf> list = kafkaInfoDao.getJobConf();
		if (list != null && list.size() > 0) {
			for (TbKafkaJobConf jobConf : list) {
				String jobStatus = jobConf.getSTATUS();
				String jobName = jobConf.getNAME();
				
				// 如果是 有效的 job, 检查是否是已经执行的job，若未执行的job 直接加入 scheduler 中
				if ("0".equals(jobStatus)) {
					if (isJobRunning(scheduler, jobName)) {
						continue;
					}
					
					logger.info("【新增Job】 " + jobName);
					
					try {
						Class<? extends Job> jobClass = (Class<? extends Job>) Class.forName(jobConf.getHANDLERCLASSNAME());
						addJob(scheduler, jobName, jobConf.getCRONEXPRESSION(), jobClass, jobConf, jobStatus);
					} catch (Exception e) {
						logger.info("新增Job failed .... ....");
					}
				}
				
				// 如果是 无效的 job, 检查是否是已经执行的job，若是执行中的job 直接从 scheduler 中 删除
				if ("1".equals(jobStatus)) {
					if (isJobRunning(scheduler,jobName)) {
						logger.info("【删除Job】 " + jobName);
						
						try {
							JobKey jobKey = new JobKey(jobName, jobName);
							scheduler.pauseJob(jobKey);
							scheduler.deleteJob(jobKey);
							
							ShareVarUtil.setJobConsumerThreadNum(jobName, 0);
							ShareVarUtil.setJobSendThreadNum(jobName, 0);
						} catch (Exception e) {
							logger.info("删除Job failed .... ....");
						}
					}
				} else {
					continue;
				}		
			}
		}	
	}
	
	****/
	
	/**
	 * 判断此任务是不是已经执行
	 * @param scheduler
	 * @param name
	 * @return
	 */
	/****
	private boolean isJobRunning(org.quartz.Scheduler scheduler,String name) {
		boolean ret = false;
		JobKey jobKey = new JobKey(name, name);
		try {
			if (scheduler.checkExists(jobKey)) {
				ret = true;
			}
		} catch (SchedulerException e) {
			ret = false;
			logger.info("job checked error .... ....");
		}
		return ret;
	}
	*****/
}
