package com.chinacreator.kafka.quartz;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import org.jboss.logging.Logger;
import org.quartz.DisallowConcurrentExecution;
import org.quartz.Job;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.quartz.PersistJobDataAfterExecution;
import org.springframework.beans.factory.annotation.Value;

import com.chinacreator.kafka.bean.KafkaBean;
import com.chinacreator.kafka.bean.SourceDataRecBean;
import com.chinacreator.kafka.bean.TbKafkaJobConf;
import com.chinacreator.kafka.mapper.KafkaInfoDao;
import com.chinacreator.kafka.mapper.SourceDataRecDao;
import com.chinacreator.kafka.service.BaseService;
import com.chinacreator.kafka.service.DouyinDeal;
import com.chinacreator.kafka.service.SyncMainDeal;
import com.chinacreator.kafka.util.JsonUtil;
import com.chinacreator.kafka.util.ParamsConfig;
import com.chinacreator.kafka.util.SpringContextUtils;

/**
 * 同步订购关系
 * @author xu.zhou
 * 对应原工程文件：/kafka-ftp/src/com/chinacreator/kafka/SyncMainNew.java
 * @date 20201217
 * 问题情景：
周六收到关于KAFKA橙卡数据在入库后没有及时生成订购关系，经查，KAFKA同步处理数据积压，大约是四五十万，按订购时间排序，橙卡数据前面有几十万月初的订购的数据正在等着处理，因此造成橙卡订购关系未能及时生成订购关系。

当前同步处理流程：
当前的处理流程是每次获取1000条数据，然后使用多线程对数据进行操作分类，分为订购和退订，再对业务进行验证，比如：是否已订购，是否已退订，是否业务互斥，
解析完成后，上面的多线程处理结束。再集中对数据进入订购关系表的入库处理，先执行订购数据再执行退订数据（使用多线程，是周六的主要修改部分，之前1000条数据入库时间是70到80秒，更新后的时间为1至5秒）。
通过调整后积压数据不到一小时全部处理完成，速度高峰期是每秒1千左右。

可能存在问题：
这种流程很可能出现在1000条数据里有号码相同，产品相同的订购和退订数据，或产品不相同但业务是互斥的数据，
由于是集中入库，之前的业务验证可能失效，应该先退订的却先执行了订购，应该先订购的却先执行了退订，应该业务互斥的却没有互斥，最终结果是订购关系紊乱。
调整之前每次只取50条数据，数据量少可能出现异常的情况相对要少些。

更新方案：
1. 取数据：保持与当前一致，每次取1000条，按操作时间排序（订购时间或退订时间，升序）。
2. 对数据中手机号码有重复进行筛选，不管产品，只要手机号码相同的，只保留操作时间最早的一条，剔除的数据在下一次再获取，这样可以保证同一手机号码的产品订购退订是按操作时间的顺序处理。
3. 对筛选后的数据进行多线程处理，每一条数据在业务验证后即实时入库，不用按订购或退订排先后（批数据中同一手机号码只有一条数据）。
4. 参数配置化：包括取数据条数，线程数等，便于调整。
 */
//保存在JobDataMap传递的参数
@PersistJobDataAfterExecution
//保证多个任务间不会同时执行
@DisallowConcurrentExecution
public class SyncMainQuartz extends BaseService implements Job {
		
	private static Logger logger = Logger.getLogger("syncmain");
	
	//订购关系同步成功后要发送短信的产品编码，多个用|分隔
	//取不到值
	//@Value("${config.smssend.spid}")
	//private String sendsmsspid;
	
	@Override
	public void execute(JobExecutionContext context)
			throws JobExecutionException {
		try {
			ParamsConfig pc = (ParamsConfig)SpringContextUtils.getBean(ParamsConfig.class);
			String sendsmsspid = pc.sendsmsspid;
			//获取配置信息
			TbKafkaJobConf jobConf = (TbKafkaJobConf)context.getJobDetail().getJobDataMap().get("data");
			KafkaInfoDao kidao = (KafkaInfoDao) SpringContextUtils.getBean(KafkaInfoDao.class);
			
			//获取要处理的数据
			List<KafkaBean> list = kidao.qrySyncData(jobConf.getPAGES());
			logger.info("获取数据条数=> " + list.size());
			//去除重复数据
			delRepeat(list);
			if(list != null && list.size()>0){
				//获取产品配置
				List<HashMap> confList = kidao.getProductConfAll();
				//产品ID配置信息
				HashMap<String, String> productmap = getProductinfo(confList);
				//批量更新为待处理状态
				kidao.updBatchRequestData(list);
				CountDownLatch threadSignal = new CountDownLatch(list.size());
				ExecutorService executorService = new ThreadPoolExecutor(jobConf.getCOREPOOLSIZE(), jobConf.getMAXIMUMPOOLSIZE(), 10L, TimeUnit.MINUTES, new LinkedBlockingQueue<Runnable>(), new ThreadPoolExecutor.CallerRunsPolicy());
				for (KafkaBean bean : list) {
					SyncMainDeal deal = new SyncMainDeal(bean, list.size(), threadSignal, logger, kidao, productmap, sendsmsspid);
					executorService.execute(deal);
				}				
				//等待多线程处理完
				executorService.shutdown();
				try {
					executorService.awaitTermination(1L, TimeUnit.MINUTES);
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
			}else{
				sleep(Long.parseLong(jobConf.getSLEEPTIME()+""),logger);
			}
			logger.info("数据处理完成");
		} catch (Exception e) {
			e.printStackTrace();
			logger.error("订购数据同步处理出现异常，"+e.getMessage());
		}	
	}
	
	/**
	 * 按手机号码去重复数据，每次处理的数据手机号码不能相同
	 * @param list
	 */
	private void delRepeat(List<KafkaBean> list){
		if(list == null || list.size() == 0){
			return;
		}
		Map<String, KafkaBean> dataMap = new HashMap<String, KafkaBean>();
		if(list != null && list.size()>0){
			logger.info("去重复前数据个数："+list.size());
			Iterator<KafkaBean> it = list.iterator();;
			KafkaBean bean = null;
			while(it.hasNext()){
				bean = it.next();
				//System.out.println(JsonUtil.objectToJson(bean));
				if(dataMap.containsKey(bean.getSerialNumber())){
					logger.info("手机号码重复，移除数据，下次处理，"+JsonUtil.objectToJson(bean));
					it.remove(); //移除对象
					continue;
				}else{
					dataMap.put(bean.getSerialNumber(), bean);
				}
			}
			logger.info("去重复后数据个数："+list.size());
		}
	}
	
	/**
	 * 获取产品ID配置信息，转化成map
	 * @param confList
	 * @return
	 */
	private HashMap<String, String> getProductinfo(List<HashMap> confList){
		HashMap<String, String> productmap = new HashMap<String, String>();
		for(HashMap hm : confList){
			productmap.put(hm.get("PRODUCTID").toString(), "{\"cpid\":\""+hm.get("CPID")+"\",\"spid\":\""+hm.get("SPID")+"\",\"mutex\":\""+(hm.get("MUTEX") == null ? "" : hm.get("MUTEX"))+"\"}");
		}
		//System.out.println("产品配置信息：productmap=>"+productmap);
		return productmap;
	}
}
