package com.chinacreator.kafka.sync.quartz;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CountDownLatch;
import org.apache.commons.lang.StringUtils;
import org.jboss.logging.Logger;
import org.springframework.transaction.PlatformTransactionManager;
import org.springframework.transaction.TransactionDefinition;
import org.springframework.transaction.TransactionStatus;
import com.chinacreator.kafka.exception.VideoException;
import com.chinacreator.kafka.mapper.ChannelSyncDao;
import com.chinacreator.kafka.sync.handle.IHandleService;
import com.chinacreator.kafka.util.SpringContextUtils;

/**
 * 内部渠道订购关系同步处理类
 * @author xu.zhou
 * @date 20220218
 */
public class ActivityHandleSyncDeal implements Runnable{
	
	private Logger logger;
	
	private ChannelSyncDao dao;
	
	private int totalSize;
	
	private CountDownLatch threadSignal;
	
	//TD_CHSYNC_WAIT表数据
	private HashMap<String, String> data;
	
	public ActivityHandleSyncDeal(HashMap<String, String> _data, int _totalSize, CountDownLatch _threadSignal,Logger _logger, ChannelSyncDao _dao) {
		this.data = _data;
		this.logger = _logger;
		this.dao = _dao;
		this.totalSize = _totalSize;
		this.threadSignal = _threadSignal;
	}
	
	@SuppressWarnings("unchecked")
	public void run(){
		HashMap<String, Object> logmap = new HashMap<String, Object>();
		long starttime = System.currentTimeMillis();
		logmap.put("data", data);
		String resultcode = "5000";
		String errorinfo = "无返回信息";
		try {			
			//更新数据为处理中
			dao.updWaitSyncing(data.get("ID"));
			
			//获取接口配置信息
			HashMap<String, String> invokeConf = getInvokeConf();
			if(invokeConf != null){
				if("2".equals(invokeConf.get("SYNCTYPE"))){//内部同步
					String handle = invokeConf.get("URL");
					//获取真正处理类
					IHandleService handleService = (IHandleService) SpringContextUtils.getBean(handle);
					//业务处理
					HashMap<String, String> reData = handleService.handle(data, invokeConf);
					resultcode = reData.get("resultcode");
					errorinfo = reData.get("errorinfo");
				}
			}
		} catch (Exception e) {
			if("无返回信息".equals(errorinfo)){
				errorinfo = "";
			}
			if (e instanceof VideoException) {
				resultcode = ((VideoException) e).getErrorcode();
				errorinfo = ((VideoException) e).getMessage()+errorinfo;
			} else {
				e.printStackTrace();
				logger.error("处理数据出现异常，"+e.getMessage(),e);	
				resultcode = "8000";
				errorinfo = "处理数据出现异常，"+e.getMessage()+errorinfo;
			}	
		} finally {
			//当前返回信息非接口文档定义格式时，resultcode为空
			if(StringUtils.isEmpty(resultcode)){
				resultcode = "5001";
			}
			logmap.put("resultcode", resultcode);
			logmap.put("errorinfo", errorinfo);
			logmap.put("times", System.currentTimeMillis() - starttime);
			//更新待同步数据处理结果
			updWaitSyncRes(resultcode, errorinfo);
			threadSignal.countDown();
			logger.info(logmap);
			//logger.info((totalSize - threadSignal.getCount()) + "/" + totalSize+"，耗时："+(System.currentTimeMillis() - starttime)+" ms"+", 处理结果："+resultcode+"=>"+errorinfo+"，id："+data.get("ID"));
		}
	}
	

    /**
     * 获取渠道接口配置信息
     * @return
     * @throws Exception
     */
    private HashMap<String, String> getInvokeConf() throws Exception{
    	HashMap<String, String> invokeConf = null;
		//获取配置信息
		List<HashMap<String, String>> dataList = dao.getInvokeConfByOrdertype(data.get("INVOKECHANNEL"), data.get("ORDERTYPE"));
		if(dataList == null || dataList.size() == 0){
			throw new VideoException("99999", "TB_CHSYNC_INVOKE_CONF无配置", new String[0]);
		}		
		invokeConf = dataList.get(0); 
		
    	return invokeConf;
    }
	
	/**
	 * 更新待同步数据处理结果
	 * @param resultcode
	 * @param errorinfo
	 */
	private void updWaitSyncRes(String resultcode, String errorinfo){
		try {
			if(errorinfo != null && errorinfo.length() > 300){
				errorinfo = errorinfo.substring(0, 300);
			}		
		    dao.updWaitSyncRes(data.get("ID"), resultcode, errorinfo);
		} catch (Exception e) {
			e.printStackTrace();
			logger.error("updWaitSyncRes更新出现异常，"+e.getMessage()+"=>"+data.get("ID"));
		} 
		
		//更新同步订购记录表的处理结果
		updOderRecRes();
	}
	
	/**
	 * 更新同步订购记录表的处理结果
	 * 所有同步成功才算成功
	 * 同步失败达到最大次数才算失败
	 * @param 
	 */
	private void updOderRecRes(){		
		//使用事务控制
		PlatformTransactionManager dataSourceTransactionManager = SpringContextUtils.getBean(PlatformTransactionManager.class);
		TransactionDefinition transactionDefinition = SpringContextUtils.getBean(TransactionDefinition.class);
		TransactionStatus transactionStatus = dataSourceTransactionManager.getTransaction(transactionDefinition);
		try {	
			//同步记录表TD_CHSYNC_ORDER_REC的ID
			String recid = data.get("RECID");
			//System.out.println("recid=>"+recid);
			//是否全部同步完成
			boolean hasDone = true;
			//是否同步成功，1待处理，2处理中，3处理失败，0成功
			String syncsucceed = "0";
			String resultcode = "";
			String synccount = "";
			//掊品返回非0，但也是成功的编码，比如华盛接口返回{"code":7018,"data":null,"message":"该用户不存在或已销户","success":false}
			//7018这些调用应该记录为同步成功
			String succcode = null;

			//接口配置信息
			List<HashMap<String, String>> confList = null;
			HashMap<String, String> invokeConf = null;
			
			List<HashMap<String, String>> dataList = dao.getWaitDataByRecid(recid);
			for(HashMap<String, String> hm : dataList){
				//获取接口配置信息
				confList = dao.getInvokeConfByOrdertype(hm.get("INVOKECHANNEL"), hm.get("ORDERTYPE"));
				if(confList != null && confList.size()>0){
					//获取配置信息
					invokeConf = dao.getInvokeConfByOrdertype(hm.get("INVOKECHANNEL"), hm.get("ORDERTYPE")).get(0);
					succcode = invokeConf.get("SUCCCODE");
				}

				if(succcode == null) succcode = "";
				
				resultcode = hm.get("RESULTCODE");
				synccount = hm.get("SYNCCOUNT");
				//System.out.println("resultcode=>"+resultcode+", synccount=>"+synccount);
				//同步未成功，99999代表无接口配置
				//1 待处理，2处理中，99999无接口配置，0成功, SYNCCOUNT为5代表已重试到最大次数
				if(!"1".equals(resultcode) 
						&& !"2".equals(resultcode)
						&& !"99999".equals(resultcode)
						&& !"0".equals(resultcode)
						&& succcode.indexOf(resultcode) == -1
						&& "5".equals(synccount)){
					syncsucceed = "3";
				}
								
				//同步未完成
				//1待处理，2处理中, 不等于99999且重试次数未达到最大次数
				if("1".equals(resultcode) 
						//待处理
						|| "2".equals(resultcode)
						//同步失败，且未达到最大次数
						|| (!"99999".equals(resultcode) && !"0".equals(resultcode) && !"5".equals(synccount))){
					hasDone = false;
				}
			}
			logger.info("id=>"+data.get("ID")+"，recid=>"+recid+"，syncsucceed=>"+syncsucceed+"，hasDone=>"+hasDone);
			//同步完成，更新同步记录表状态，并更新待处理表的是否更新同步记录表状态（用于数据迁移）
			if(hasDone){
				//更新同步记录表状态为成功或失败
				dao.updOrderRecRes(recid, syncsucceed);
				//根据SYNCID更新对应同步数据
				dao.updWaitUpdrelByRecid(recid);
			}
			//提交事务
			dataSourceTransactionManager.commit(transactionStatus);
		} catch (Exception e) {
			e.printStackTrace();
			//回滚事务
			dataSourceTransactionManager.rollback(transactionStatus);
			logger.error("updOderRecRes更新出现异常，"+e.getMessage()+"=>"+data.get("ID"),e);
		} 
	}
	

    
    /**
     * 输出调接口日志
     * @param invokeurl
     * @param inputparams
     * @param outputparams
     */
    private void printLog(HashMap<String, String> invokeConf, String inputparams, String outputparams){
    	logger.info(data.get("ID")+"<=>"+data.get("USERID")+"=>接口配置："+invokeConf+"，请求报文:【"+inputparams+"】响应报文：【"+outputparams+"】");
    }
}
